from .Autocad.AcadEntity import *
from .Autocad.AcadObject import *
from .Autocad.Entities import *
from .Autocad.Objects import *
from .Types import *
from .Autocad.Proxy import *
from abc import ABC, abstractmethod


class AutoCAD(AcadApplication):
    _ACAD_TYPE_MAP = {
        'AcDbFace':Acad3DFace, # 3D-грань
        'AcDbPolygonMesh':AcadPolygonMesh, # Полигональная сеть
        'AcDb3dPolyline':Acad3DPolyline, # 3D-полилиния
        'AcDbArc':AcadArc, #Дуга
        'AcDbAttributeDefinition':AcadAttribute, # Определение атрибута
        'AcDb3dSolid':Acad3DSolid, # 3D-тело
        'AcDbCircle':AcadCircle, # Круг
        'AcDb3PointAngularDimension':AcadDim3PointAngular, # Угловой размер (3 точки)
        'AcDbAlignedDimension':AcadDimAligned, # Параллельный размер
        'AcDbDiametricDimension':AcadDimDiametric, # Диаметр
        'AcDbOrdinateDimension':AcadDimOrdinate, # Ординатный размер
        'AcDbRadialDimension':AcadDimRadial, # Размер радиуса
        'AcDbRotatedDimension':AcadDimRotated, # Повернутый размер
        'AcDbEllipse':AcadEllipse, # Эллипс
        'AcDbHatch':AcadHatch, # Штриховка
        'AcDbLeader':AcadLeader, # Выноска
        'AcDbMText':AcadMtext, # МТекст
        'AcDbText':AcadText, # Текст
        'AcDbPolyline':AcadLWPolyline, # Полилиния
        'AcDbLine':AcadLine, # Отрезок
        'AcDbMInsertBlock':AcadMInsertBlock, # Мн-блок
        'AcDbMLeader':AcadMLeader, # Мультивыноска
        'AcDbMline':AcadMLine, # МЛиния
        'AcDbPoint':AcadPoint, # Точка
        'AcDbPolyFaceMesh':AcadPolyfaceMesh, # Многогранная сеть
        'AcDb2dPolyline':AcadPolyline, # 2D-полилиния
        'AcDbRasterImage':AcadRasterImage, # Растровое изображение
        'AcDbRay':AcadRay, # Луч
        'AcDbRegion':AcadRegion, # Область
        'AcDbSection':AcadSection, # Объект сечения
        'AcDbShape':AcadShape, # Форма
        'AcDbSolid':AcadSolid, # Тело
        'AcDbSpline':AcadSpline, # Сплайн
        'AcDbTable':AcadTable, # Таблица
        'AcDbFcf':AcadTolerance, # Допуск
        'AcDbTrace':AcadTrace, # Полоса
        'AcDbXline':AcadXline, # Прямая
        'AcDbWipeout':AcadWipeout,
        'AcDbXrecord':AcadXRecord
        # 'AcDbBlockReference':(AcadExternalReference, AcadBlockReference), ##
    }

    def __init__(self):
        super().__init__(dispatch_object = None)

    def uGetAcadAcCmColor(self) -> AcadAcCmColor:
        progID = AcadApplication.__app_full_name__.replace('Application','AcCmColor')
        obj = self.GetInterfaceObject(progID)
        return AcadAcCmColor(obj)

    def uGetAcadLayerStateManager(self) -> AcadLayerStateManager:
        progID = AcadApplication.__app_full_name__.replace('Application','AcadLayerStateManager')
        obj = self.GetInterfaceObject(progID)
        return AcadLayerStateManager(obj)

    def uGetAcadSecurityParams(self) -> AcadSecurityParams:
        progID = AcadApplication.__app_full_name__.replace('Application','SecurityParams')
        obj = self.GetInterfaceObject(progID)
        return AcadSecurityParams(obj)

    def uSetXData(obj: AcadObject, xdm: XDataManager) -> None: 
        try:
            obj.Document.RegisteredApplications.Item(xdm.RegAppName)
        except:
            obj.Document.RegisteredApplications.Add(xdm.RegAppName)
        obj.SetXData(xdm.xDataType, xdm.xDataValue)

    def uGetObjectType(self, obj: AcadObject) -> type:
        return self._ACAD_TYPE_MAP.get(obj.ObjectName,None)



class BlockReference(ABC):
    def __init__(self, acad_block_reference:AcadBlockReference = None):
        self.acad_block_reference:AcadBlockReference = acad_block_reference
        self.Attributes = self._get_block_attributes(acad_block_reference)
        self.DynamicProperties = self._get_block_dynamic_properties(acad_block_reference)
    
    @staticmethod
    def _get_block_attributes(block_ref:AcadBlockReference):
        attrib = None
        if isinstance(block_ref, AcadBlockReference) and block_ref.HasAttributes:
            obj_arr = block_ref.GetAttributes()
            attrib:list[AcadAttributeReference] = []
            for attr in obj_arr:
                attrib.append(AcadAttributeReference(attr))
        return attrib

    @staticmethod
    def _get_block_dynamic_properties(block_ref:AcadBlockReference):
        dynprop = None
        if isinstance(block_ref, AcadBlockReference) and block_ref.IsDynamicBlock:
            dyn_arr = block_ref.GetDynamicBlockProperties()
            dynprop:list[AcadDynamicBlockReferenceProperty] = []
            for dattr in dyn_arr:
                dynprop.append(AcadDynamicBlockReferenceProperty(dattr))
        return dynprop

    @classmethod
    def get_list(cls, space: AcadModelSpace | AcadPaperSpace):
        for v in space:
            blk_ref = AcadBlockReference(v)
            if blk_ref.ObjectName == 'AcDbBlockReference' and blk_ref.EffectiveName == cls.BlockName:
                yield cls(blk_ref)

    @property
    @abstractmethod
    def BlockName(self) -> str: ...

    def insert(self, insertion_point:PyGePoint3d, space: AcadModelSpace | AcadPaperSpace | AcadBlock):
        block = space.InsertBlock(insertion_point, self.BlockName)
        if self.acad_block_reference:
            return type(self)(block)

        self.Attributes = self._get_block_attributes(block)
        self.DynamicProperties = self._get_block_dynamic_properties(block)
        self.acad_block_reference = block

    
    def attribute(self, tag_name: str) -> AcadAttributeReference:
        if self.acad_block_reference and self.Attributes:
            for attr in self.Attributes:
                if attr.TagString == tag_name:
                    return attr
    
    def dynamic_property(self, dyn_name: str) -> AcadDynamicBlockReferenceProperty:
        if self.acad_block_reference and self.DynamicProperties:
            for dyn in self.DynamicProperties:
                if dyn.PropertyName == dyn_name:
                    return dyn

    def get_attribute_value(self, tag_name) -> str:
        attr = self.attribute(tag_name)
        if attr: return attr.TextString

    def set_attribute_value(self, tag_name, value):
        attr = self.attribute(tag_name)
        if attr: attr.TextString = value
    
    def get_dynamic_property_value(self, dyn_name:str) -> Variant:
        dyn = self. dynamic_property(dyn_name)
        if dyn: return dyn.Value

    def set_dynamic_property_value(self, dyn_name, value):
        dyn = self. dynamic_property(dyn_name)
        if dyn: dyn.Value = value


from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadRasterImage(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    Brightness: int = proxy_property(int,'Brightness',AccessMode.ReadWrite)
    ClippingEnabled: bool = proxy_property(bool,'ClippingEnabled',AccessMode.ReadWrite)
    Contrast: int = proxy_property(int,'Contrast',AccessMode.ReadWrite)
    Fade: int = proxy_property(int,'Fade',AccessMode.ReadWrite)
    Height: float = proxy_property(float,'Height',AccessMode.ReadOnly)
    ImageFile: str = proxy_property(str,'ImageFile',AccessMode.ReadWrite)
    ImageHeight: float = proxy_property(float,'ImageHeight',AccessMode.ReadWrite)
    ImageVisibility: bool = proxy_property(bool,'ImageVisibility',AccessMode.ReadWrite)
    ImageWidth: float = proxy_property(float,'ImageWidth',AccessMode.ReadWrite)
    Name: str = proxy_property(str,'Name',AccessMode.ReadWrite)
    Origin: PyGePoint3d = proxy_property('PyGePoint3d','Origin',AccessMode.ReadWrite)
    Rotation: float = proxy_property(float,'Rotation',AccessMode.ReadWrite)
    ScaleFactor: float = proxy_property(float,'ScaleFactor',AccessMode.ReadWrite)
    ShowRotation: bool = proxy_property(bool,'ShowRotation',AccessMode.ReadWrite)
    Transparency: bool = proxy_property(bool,'Transparency',AccessMode.ReadWrite)
    Width: float = proxy_property(float,'Width',AccessMode.ReadOnly)

    def ClipBoundary(self, PointsArray: PyGePoint2dArray) -> None:
        self._obj.ClipBoundary(PointsArray())

    def Copy(self) -> AcadRasterImage:
        return AcadRasterImage(self._obj.Copy())




from __future__ import annotations
from ..Base import *
from ..AcadObject import *
from ..Proxy import *
from .AcadState import AcadState


class AcadApplication(Application, AppObject):
    _first_instance_initialized = False
    __app_name__ = 'AutoCAD'

    def _manage_application_instance(self) -> CDispatch:
        if not AcadApplication._first_instance_initialized:
            clsid = get_clsid(self)
            AcadApplication.__app_clsid__ = clsid[0]
            AcadApplication.__app_full_name__ = clsid[1]
            acad_app = com_server_is_running(AcadApplication.__app_full_name__)
            self._is_owner = not bool(acad_app)
            if not acad_app:
                acad_app = AppCreate(AcadApplication.__app_full_name__)
        else:
            acad_app = create_new_instance_explicitly(AcadApplication.__app_full_name__)
            self._is_owner = True
        AcadApplication._first_instance_initialized = True
        return acad_app

    ActiveDocument: AcadDocument = proxy_property('AcadDocument','ActiveDocument',AccessMode.ReadOnly)
    Application: 'AcadApplication' = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    Caption: str = proxy_property(str,'Caption', AccessMode.ReadOnly)
    Documents: AcadDocuments = proxy_property('AcadDocuments','Documents',AccessMode.ReadOnly)
    FullName: str = proxy_property(str,'FullName',AccessMode.ReadOnly)
    Height: float = proxy_property(float,'Height',AccessMode.ReadWrite)
    HWND: int = proxy_property(int,'HWND',AccessMode.ReadOnly)
    LocaleId: int = proxy_property(int,'LocaleId',AccessMode.ReadOnly)
    MenuBar: AcadMenuBar = proxy_property('AcadMenuBar','MenuBar',AccessMode.ReadOnly)
    MenuGroups: AcadMenuGroups = proxy_property('AcadMenuGroups','MenuGroups',AccessMode.ReadOnly)
    Name: str = proxy_property(str,'Name',AccessMode.ReadOnly)
    Path: str = proxy_property(str,'Path',AccessMode.ReadOnly)
    Preferences: AcadPreferences = proxy_property('AcadPreferences','Preferences',AccessMode.ReadOnly)
    VBE: AppObject = proxy_property('AppObject','VBE',AccessMode.ReadOnly)
    Version: str = proxy_property(str,'Version',AccessMode.ReadOnly)
    Visible: bool = proxy_property(bool,'Visible',AccessMode.ReadWrite)
    Width: float = proxy_property(float,'Width',AccessMode.ReadWrite)
    WindowLeft: int = proxy_property(int,'WindowLeft',AccessMode.ReadWrite)
    WindowState: int = proxy_property(int,'WindowState',AccessMode.ReadWrite)
    WindowTop: int = proxy_property(int,'WindowTop',AccessMode.ReadWrite)

    def StatusID(self, VportObj: AcadViewport) -> bool: 
        return self._obj.StatusId(VportObj())

    def Eval(self, Expression: str) -> None: 
        self._obj.Eval(Expression)

    def GetAcadState(self) -> AcadState: 
        return AcadState(self._obj.GetAcadState())

    def GetInterfaceObject(self, ProgID: str) -> AppObject: 
        return AppObject(self._obj.GetInterfaceObject(ProgID))

    def ListARX(self) -> vStringArray: 
        return vStringArray(self._obj.ListArx())

    def LoadARX(self, Name) -> None: 
        self._obj.LoadArx(Name)

    def LoadDVB(self, Name) -> None: 
        self._obj.LoadDVB(Name)

    def Quit(self) -> None: 
        self._obj.Quit()

    def RunMacro(self, MacroPath: str) -> None: 
        self._obj.RunMacro(MacroPath)

    def UnloadARX(self, Name: str) -> None:
        self._obj.UnloadArx(Name)

    def UnloadDVB(self, Name: str) -> None: 
        self._obj.UnloadDVB(Name)

    def Update(self) -> None: 
        self._obj.Update()

    def ZoomAll(self) -> None: 
        self._obj.ZoomAll()

    def ZoomCenter(self, Center: PyGePoint3d, Magnify: float) -> None: 
        self._obj.ZoomCenter(Center(), Magnify)

    def ZoomExtents(self) -> None: 
        self._obj.ZoomExtents()

    def ZoomPickWindow(self) -> None: 
        self._obj.ZoomPickWindow()

    def ZoomPrevious(self) -> None: 
        self._obj.ZoomPrevious()

    def ZoomScaled(self, Scale: float, ScaleType: AcZoomScaleType) -> None: 
        self._obj.ZoomScaled(Scale, ScaleType)

    def ZoomWindow(self, LowerLeft: PyGePoint3d, UpperRight: PyGePoint3d) -> None: 
        self._obj.ZoomWindow(LowerLeft(), UpperRight())


from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class Acad3DSolid(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    Centroid: PyGePoint2d = proxy_property('PyGePoint2d','Centroid',AccessMode.ReadOnly)
    History: bool = proxy_property(bool,'History',AccessMode.ReadWrite)
    MomentOfInertia: PyGePoint3d = proxy_property('PyGePoint3d','MomentOfInertia',AccessMode.ReadOnly)
    Position: PyGePoint3d = proxy_property('PyGePoint3d','Position',AccessMode.ReadOnly)
    PrincipalDirections: PyGeVector3d = proxy_property('PyGeVector3d','PrincipalDirections',AccessMode.ReadOnly)
    PrincipalMoments: PyGeVector3d = proxy_property('PyGeVector3d','PrincipalMoments',AccessMode.ReadOnly)
    ProductOfInertia: PyGeVector3d = proxy_property('PyGeVector3d','ProductOfInertia',AccessMode.ReadOnly)
    RadiiOfGyration: PyGeVector3d = proxy_property('PyGeVector3d','RadiiOfGyration',AccessMode.ReadOnly)
    ShowHistory: bool = proxy_property(bool,'ShowHistory',AccessMode.ReadWrite)
    SolidType: str = proxy_property(str,'SolidType',AccessMode.ReadWrite)
    Volume: float = proxy_property(float,'Volume',AccessMode.ReadOnly)
    
    def Boolean(self, Operation: AcBooleanType, Object: Acad3DSolid | AcadRegion) -> None:
        self._obj.Boolean(Operation.value, Object())
        
    def CheckInterference(self, Object: Acad3DSolid, CreateInterferenceSolid: bool) -> bool:
        SolidsInterfere = self._obj.CheckInterference(Object(), CreateInterferenceSolid)
        return SolidsInterfere

    def Delete(self) -> None:
        self._obj.Delete()

    def Copy(self) -> Acad3DSolid:
        return Acad3DSolid(self._obj.Copy())
        
    def SectionSolid(self, Point1: PyGePoint3d, Point2: PyGePoint3d, Point3: PyGePoint3d) -> AcadRegion:
        return AcadRegion(self._obj.SectionSolid(Point1(), Point2(), Point3()))

    def SliceSolid(self, Point1: PyGePoint3d, Point2: PyGePoint3d, Point3: PyGePoint3d, Negative: bool) -> Acad3DSolid:
        return Acad3DSolid(self._obj.SliceSolid(Point1(), Point2(), Point3(), Negative))



from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadXline(AcadEntity):
    def __init__(self, obj) -> None: ...
    BasePoint: PyGePoint3d
    DirectionVector: PyGeVector3d
    SecondPoint: PyGePoint3d
    def Copy(self) -> AcadXline: ...



from ..Base import *
from ..Proxy import *

class AcadDatabasePreferences(AppObject):
    def __init__(self, obj) -> None: ...
    AllowLongSymbolNames: bool
    Application: AcadApplication
    ContourLinesPerSurface: int
    DisplaySilhouette: bool
    Lineweight: AcLineWeight
    LineweightDisplay: bool
    ObjectSortByPlotting: bool
    ObjectSortByPSOutput: bool
    ObjectSortByRedraws: bool
    ObjectSortByRegens: bool
    ObjectSortBySelection: bool
    ObjectSortBySnap: bool
    OLELaunch: bool
    RenderSmoothness: float
    SegmentPerPolyline: int
    SolidFill: bool
    TextFrameDisplay: bool
    XRefEdit: bool
    XRefLayerVisibility: bool



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadMLine(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    Coordinates: PyGePoint3d = proxy_property('PyGePoint3d','Coordinates',AccessMode.ReadWrite)
    Justification: AcMLineJustification = proxy_property('AcMLineJustification','Justification',AccessMode.ReadWrite)
    MLineScale: float = proxy_property(float,'MLineScale',AccessMode.ReadWrite)
    StyleName: str = proxy_property(str,'StyleName',AccessMode.ReadOnly)

    def Copy(self) -> AcadMtext:
        return AcadMtext(self._obj.Copy())


from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *
from enum import IntEnum

class AcadUtility(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    def AngleFromXAxis(self, Point1: PyGePoint3d, Point2: PyGePoint3d) -> float:
        return self._obj.AngleFromXAxis(Point1(), Point2())
        
    def AngleToReal(self, Angle: str, Unit: AcAngleUnits) -> float:
        return self._obj.AngleToReal(Angle, Unit)
        
    def AngleToString(self, Angle: float, Unit: AcAngleUnits, Precision: int) -> str:
        return self._obj.AngleToString(Angle, Unit, Precision)
        
    def CreateTypedArray(self, Type: VbVarType, Value1, *args):
        VarArr = vObjectEmpty
        self._obj.CreateTypedArray(VarArr, Type, Value1, *args)
        return VarArr
        
    def DistanceToReal(self, Distance: str, Unit: AcUnits) -> float:
        return self._obj.DistanceToReal(Distance, Unit)

    def GetAngle(self, Point: PyGePoint3d = None, Prompt: str = '') -> float:
        if Point is None:
            return self._obj.GetAngle()
        return self._obj.GetAngle(Point(),Prompt)

    def GetCorner(self, Point: PyGePoint3d, Prompt: str = '') -> tuple:
        return self._obj.GetCorner(Point(), Prompt)
        
    def GetDistance(self, Point: PyGePoint3d = None, Prompt: str = None) -> float:
        return self._obj.GetDistance(Point(), Prompt)
        
    def GetEntity(self, Prompt: str = '') -> tuple: #Object: Out[AppObject], PickedPoint: Out[VARIANT], 
        Object, PickedPoint = self._obj.GetEntity()
        return Object, PickedPoint
        
    def GetInput(self) -> str: 
        return self._obj.GetInput()

    def GetInteger(self, Prompt: str = '') -> int: 
        return self._obj.GetInteger(Prompt)
        
    def GetKeyword(self, Prompt: str = '') -> str: 
        return self._obj.GetKeyword(Prompt)

    def GetObjectIdString(self, acadObject: AcadObject, bHex: bool) -> str: 
        return self._obj.GetObjectIdString(acadObject(), int(bHex))
        
    def GetOrientation(self, Point: PyGePoint3d, Prompt: str = '') -> float: 
        return self._obj.GetOrientation(Point(), Prompt)
        
    def GetPoint(self, Point: PyGePoint3d = None, Prompt: str = '') -> tuple: 
        if Point is None:
            return PyGePoint3d(self._obj.GetPoint())
        return PyGePoint3d(self._obj.GetPoint(Point, Prompt))

    def GetReal(self, Prompt: str = '') -> float: 
        return self._obj.GetReal(Prompt)
        
    def GetRemoteFile(self, URL: str, IgnoreCache: bool) -> None:
        LocalFile: str = ''
        self._obj.GetRemoteFile(URL, LocalFile, IgnoreCache)
        return LocalFile

    def GetString(self, HasSpaces : int, Prompt: str = '') -> float: 
        return self._obj.GetString(HasSpaces, Prompt)
        
    def GetSubEntity(self, Prompt: str = '') -> tuple: 
        Object = None
        PickedPoint = None
        TransMatrix = None
        ContextData = None
        Object, PickedPoint, TransMatrix, ContextData = self._obj.GetSubEntity()
        return Object, PickedPoint, TransMatrix, ContextData

    def InitializeUserInput(self, Bits : int, Keyword: str = None) -> None: 
        if Keyword:
            self._obj.InitializeUserInput(Bits, Keyword)
        else:
            self._obj.InitializeUserInput(Bits)

    def IsRemoteFile(self, LocalFile : str, URL: str) -> bool: 
        return self._obj.IsRemoteFile(LocalFile, URL)

    def IsURL(self, URL: str) -> bool: 
        return self._obj.IsURL(URL)

    def LaunchBrowserDialog(self, DialogTitle: str, OpenButtonCaption: str, StartPageURL: str, RegistryRootKey: str, OpenButtonAlwaysEnabled: bool) -> bool: 
        SelectedURL:str = ''
        return self._obj.LaunchBrowserDialog(SelectedURL, DialogTitle, OpenButtonCaption, StartPageURL, RegistryRootKey, OpenButtonAlwaysEnabled)
        return SelectedURL

    def PolarPoint(self, Point: PyGePoint3d, Angle: float, Distance: float) -> PyGePoint3d: 
        return PyGePoint3d(self._obj.PolarPoint(Point(), Angle, Distance))

    def Prompt(self, Message: str) -> None: 
        self._obj.Prompt(Message)

    def PutRemoteFile(self, URL: str, LocalFile: str) -> None: 
        self._obj.PutRemoteFile(URL, LocalFile)
        
    def RealToString(self, Value: float, Unit: AcUnits, Precision: int) -> str: 
        return self._obj.RealToString(Value, Unit, Precision)
        
    def SendModelessOperationEnded(self) -> str: 
        Context = self._obj.SendModelessOperationEnded()
        return Context
        
    def SendModelessOperationStart(self, Context: str) -> None: 
        self._obj.SendModelessOperationStart(Context)
        
    def TranslateCoordinates(self, Point: PyGePoint3d, FromCoordSystem: AcCoordinateSystem, ToCoordSystem: AcCoordinateSystem, Displacement: int, OCSNormal: PyGeVector3d = None) -> tuple: 
        if OCSNormal:
            return self._obj.TranslateCoordinates(Point(), FromCoordSystem, ToCoordSystem, Displacement, OCSNormal())
        return self._obj.TranslateCoordinates(Point(), FromCoordSystem, ToCoordSystem, Displacement, OCSNormal)

class VbVarType(IntEnum):
    vbBoolean = 11
    vbInteger = 2
    vbLong = 3
    vbSingle = 4
    vbDouble = 5
    # vbEmpty = 0
    # vbNull = 1
    # vbString = 8
    # vbObject = 9
    # vbArray = 8192
    # vbCurrency = 6
    # vbDate = 7
    # vbError = 10


from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadSurface(AcadEntity):
    def __init__(self, obj) -> None: ...
    EdgeExtensionDistances: float
    MaintainAssociativity: int
    ShowAssociativity: bool
    SurfaceType: str
    SurfTrimAssociativity: bool
    UIsolineDensity: int
    VIsolineDensity: int
    WireframeType: AcWireframeType
    def Copy(self) -> AcadSurface: ...



from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadHatch(AcadEntity):
    def __init__(self, obj) -> None: ...
    Area: float
    AssociativeHatch: bool
    BackgroundColor: AcadAcCmColor
    Elevation: float
    GradientAngle: float
    GradientCentered: bool
    GradientColor1: AcadAcCmColor
    GradientColor2: AcadAcCmColor
    GradientName: str
    HatchObjectType: AcHatchObjectType
    HatchStyle: AcHatchStyle
    ISOPenWidth: AcISOPenWidth
    Normal: PyGeVector3d
    NumberOfLoops: int
    Origin: PyGePoint3d
    PatternAngle: float
    PatternDouble: bool
    PatternName: str
    PatternScale: float
    PatternSpace: float
    PatternType: AcPatternType
    def AppendInnerLoop(self, Loop: vObjectArray[AcadArc | AcadCircle | AcadEllipse | AcadLine | AcadPolyline | AcadRegion | AcadSpline]) -> None: ...
    def AppendOuterLoop(self, Loop: vObjectArray[AcadArc | AcadCircle | AcadEllipse | AcadLine | AcadPolyline | AcadRegion | AcadSpline]) -> None: ...



from .VarType import *
from .Ac import AdeskDxfCode as AdeskDxfCode
from enum import IntEnum

class DxfGroupXDCode(IntEnum):
    kDxfXdAsciiString = ...
    kDxfRegAppName = ...
    kDxfXdControlString = ...
    kDxfXdLayerName = ...
    kDxfXdBinaryChunk = ...
    kDxfXdHandle = ...
    kDxfXdXCoord = ...
    kDxfXdWorldXCoord = ...
    kDxfXdWorldXDisp = ...
    kDxfXdWorldXDir = ...
    kDxfXdReal = ...
    kDxfXdDist = ...
    kDxfXdScale = ...
    kDxfXdInteger16 = ...
    kDxfXdInteger32 = ...

class XDataManager:
    def __init__(self, regAppName: str = '') -> None: ...
    @property
    def RegAppName(self) -> str: ...
    @RegAppName.setter
    def RegAppName(self, value: str) -> str: ...
    @property
    def xDataType(self) -> vShortArray: ...
    @property
    def xDataValue(self) -> vVariantArray: ...
    def add_data(self, group_code: DxfGroupXDCode | list[DxfGroupXDCode] | tuple[DxfGroupXDCode], value: int | str | float | list[int | str | float] | tuple[int | str | float]): ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, value) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self): ...



from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *



class AcadGroups(IAcadObjectCollection):
    def __init__(self, obj) -> None: super().__init__(obj)

    def Add(self, Name: str) -> AcadGroup: 
        return AcadGroup(self._obj.Add(Name))

    def Item(self, Index: int | str) -> AcadGroup:
        return AcadGroup(self._obj.Item(Index))

    def __iter__(self):
        for item in self._obj:
            obj = AcadGroup(item)
            yield obj


from __future__ import annotations
from ..Base import *
from ..Proxy import *
from .AcadPopupMenu import *

class AcadPopupMenuItem(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    Caption: str = proxy_property(str,'Caption',AccessMode.ReadOnly)
    Check: bool = proxy_property(bool,'Check',AccessMode.ReadWrite)
    Enable: bool = proxy_property(bool,'Enable',AccessMode.ReadWrite)
    EndSubMenuLevel: int = proxy_property(int,'EndSubMenuLevel',AccessMode.ReadWrite)
    HelpString: str = proxy_property(str,'HelpString',AccessMode.ReadWrite)
    Index: str = proxy_property(str,'Index',AccessMode.ReadOnly)
    Label: str = proxy_property(str,'Label',AccessMode.ReadWrite)
    Parent: AppObject = proxy_property('AppObject','Macro',AccessMode.ReadWrite)
    SubMenu: AcadPopupMenu = proxy_property('AcadPopupMenu','SubMenu',AccessMode.ReadOnly)
    TagString: str = proxy_property(str,'TagString',AccessMode.ReadWrite)
    Type: AcMenuItemType = proxy_property('AcMenuItemType','Type',AccessMode.ReadOnly)

    def Delete(self) -> None: 
        return self._obj.Delete()


import abc
from abc import ABC
from pythoncom import CoInitialize as CoInitialize, CoUninitialize as CoUninitialize

def com_server_is_running(prog_id: str): ...
def get_clsid(cls): ...
def create_new_instance_explicitly(clsid): ...

class AppObject:
    def __init__(self, obj) -> None: ...
    def __getattr__(self, name): ...
    def __call__(self): ...

class Application(ABC, metaclass=abc.ABCMeta):
    def __new__(cls, dispatch_object=None): ...
    def __init__(self, dispatch_object=None) -> None: ...

class AppObjectCollection(AppObject):
    def __init__(self, obj) -> None: ...
    @property
    def Count(self) -> int: ...
    def Item(self, Index: int | str) -> AppObject: ...
    def __iter__(self): ...

def clear_com_cache() -> None: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from .AcadPlotConfiguration import *



class AcadLayout(AcadPlotConfiguration):
    def __init__(self, obj) -> None: super().__init__(obj)

    Block: AcadBlock = proxy_property('AcadBlock','Block',AccessMode.ReadOnly)
    TabOrder: int = proxy_property(int,'TabOrder',AccessMode.ReadWrite)



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadAttribute(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    Alignment: AcAlignment = proxy_property('AcAlignment','Alignment',AccessMode.ReadWrite)
    Backward: bool = proxy_property(bool,'Backward',AccessMode.ReadWrite)
    Constant: bool = proxy_property(bool,'Constant',AccessMode.ReadWrite)
    FieldLength: int = proxy_property(int,'FieldLength',AccessMode.ReadWrite)
    Height: float = proxy_property(float,'Height',AccessMode.ReadWrite)
    InsertionPoint: PyGePoint3d = proxy_property('PyGePoint3d','InsertionPoint',AccessMode.ReadWrite)
    Invisible: bool = proxy_property(bool,'InsertionPoint',AccessMode.ReadWrite)
    LockPosition: bool = proxy_property(bool,'LockPosition',AccessMode.ReadWrite)
    Mode: AcAttributeMode = proxy_property('AcAttributeMode','Mode',AccessMode.ReadWrite)
    MTextAttribute: bool = proxy_property(bool,'MTextAttribute',AccessMode.ReadWrite)
    MTextAttributeContent: str = proxy_property(str,'MTextAttributeContent',AccessMode.ReadWrite)
    MTextBoundaryWidth: float = proxy_property(float,'MTextBoundaryWidth',AccessMode.ReadWrite)
    MTextDrawingDirection: AcDrawingDirection = proxy_property('AcDrawingDirection' ,'MTextDrawingDirection',AccessMode.ReadWrite)
    Normal: PyGeVector3d = proxy_property('PyGeVector3d','Normal',AccessMode.ReadWrite)
    ObliqueAngle: float = proxy_property(float,'ObliqueAngle',AccessMode.ReadWrite)
    Preset: bool = proxy_property(bool ,'Preset',AccessMode.ReadWrite)
    PromptString: str = proxy_property(str,'PromptString',AccessMode.ReadWrite)
    Rotation: float = proxy_property(float,'Rotation',AccessMode.ReadWrite)
    ScaleFactor: float = proxy_property(float,'ScaleFactor',AccessMode.ReadWrite)
    StyleName: str = proxy_property(str,'StyleName',AccessMode.ReadWrite)
    TagString: str = proxy_property(str,'TagString',AccessMode.ReadWrite)
    TextAlignmentPoint: PyGePoint3d = proxy_property('PyGePoint3d','TextAlignmentPoint',AccessMode.ReadWrite)
    TextGenerationFlag: AcTextGenerationFlag = proxy_property('AcTextGenerationFlag','TextGenerationFlag',AccessMode.ReadWrite)
    TextString: str = proxy_property(str,'TextString',AccessMode.ReadWrite)
    Thickness: float = proxy_property(float,'Thickness',AccessMode.ReadWrite)
    UpsideDown: bool = proxy_property(bool,'UpsideDown',AccessMode.ReadWrite)
    Verify: bool = proxy_property(bool,'Verify',AccessMode.ReadWrite)

    def Copy(self) -> AcadAttribute:
        return AcadAttribute(self._obj.Copy())

    def UpdateMTextAttribute(self) -> None:
        self._obj.UpdateMTextAttribute()



from ..Base import *
from ..Proxy import *
from ..AcadObject import *

class AcadUtility(AppObject):
    def __init__(self, obj) -> None: ...
    def AngleFromXAxis(self, Point1: PyGePoint3d, Point2: PyGePoint3d) -> float: ...
    def AngleToReal(self, Angle: str, Unit: AcAngleUnits) -> float: ...
    def AngleToString(self, Angle: float, Unit: AcAngleUnits, Precision: int) -> str: ...
    def CreateTypedArray(self, Type: VbVarType, Value1, *args): ...
    def DistanceToReal(self, Distance: str, Unit: AcUnits) -> float: ...
    def GetAngle(self, Point: PyGePoint3d = None, Prompt: str = '') -> float: ...
    def GetCorner(self, Point: PyGePoint3d, Prompt: str = '') -> tuple: ...
    def GetDistance(self, Point: PyGePoint3d = None, Prompt: str = None) -> float: ...
    def GetEntity(self, Prompt: str = '') -> tuple: ...
    def GetInput(self) -> str: ...
    def GetInteger(self, Prompt: str = '') -> int: ...
    def GetKeyword(self, Prompt: str = '') -> str: ...
    def GetObjectIdString(self, acadObject: AcadObject, bHex: bool) -> str: ...
    def GetOrientation(self, Point: PyGePoint3d, Prompt: str = '') -> float: ...
    def GetPoint(self, Point: PyGePoint3d = None, Prompt: str = '') -> tuple: ...
    def GetReal(self, Prompt: str = '') -> float: ...
    def GetRemoteFile(self, URL: str, IgnoreCache: bool) -> None: ...
    def GetString(self, HasSpaces: int, Prompt: str = '') -> float: ...
    def GetSubEntity(self, Prompt: str = '') -> tuple: ...
    def InitializeUserInput(self, Bits: int, Keyword: str = None) -> None: ...
    def IsRemoteFile(self, LocalFile: str, URL: str) -> bool: ...
    def IsURL(self, URL: str) -> bool: ...
    def LaunchBrowserDialog(self, DialogTitle: str, OpenButtonCaption: str, StartPageURL: str, RegistryRootKey: str, OpenButtonAlwaysEnabled: bool) -> bool: ...
    def PolarPoint(self, Point: PyGePoint3d, Angle: float, Distance: float) -> PyGePoint3d: ...
    def Prompt(self, Message: str) -> None: ...
    def PutRemoteFile(self, URL: str, LocalFile: str) -> None: ...
    def RealToString(self, Value: float, Unit: AcUnits, Precision: int) -> str: ...
    def SendModelessOperationEnded(self) -> str: ...
    def SendModelessOperationStart(self, Context: str) -> None: ...
    def TranslateCoordinates(self, Point: PyGePoint3d, FromCoordSystem: AcCoordinateSystem, ToCoordSystem: AcCoordinateSystem, Displacement: int, OCSNormal: PyGeVector3d = None) -> tuple: ...

class VbVarType(IntEnum):
    vbBoolean = 11
    vbInteger = 2
    vbLong = 3
    vbSingle = 4
    vbDouble = 5



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadLoftedSurface(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    Closed: bool = proxy_property(bool,'Closed',AccessMode.ReadWrite)
    EdgeExtensionDistances: Variant = proxy_property(Variant,'EdgeExtensionDistances',AccessMode.ReadWrite)
    EndDraftAngle: float = proxy_property(float,'EndDraftAngle',AccessMode.ReadWrite)
    EndDraftMagnitude: int = proxy_property(int,'EndDraftMagnitude',AccessMode.ReadWrite)
    EndSmoothContinuity: int = proxy_property(int,'EndSmoothContinuity',AccessMode.ReadWrite)
    EndSmoothMagnitude: float = proxy_property(float,'EndSmoothMagnitude',AccessMode.ReadWrite)
    MaintainAssociativity: int = proxy_property(int,'MaintainAssociativity',AccessMode.ReadWrite)
    NumCrossSections: int = proxy_property(int,'NumCrossSections',AccessMode.ReadWrite)
    NumGuidePaths: int = proxy_property(int,'NumGuidePaths',AccessMode.WriteOnly)
    Periodic: bool = proxy_property(bool,'Periodic',AccessMode.ReadWrite)
    ShowAssociativity: bool = proxy_property(bool,'ShowAssociativity',AccessMode.ReadWrite)
    StartDraftAngle: float = proxy_property(float,'StartDraftAngle',AccessMode.ReadWrite)
    StartDraftMagnitude: int = proxy_property(int,'StartDraftMagnitude',AccessMode.ReadWrite)
    StartSmoothContinuity: int = proxy_property(int,'StartSmoothContinuity',AccessMode.ReadWrite)
    StartSmoothMagnitude: float = proxy_property(float,'StartSmoothMagnitude',AccessMode.ReadWrite)
    SurfaceNormals: int = proxy_property(int,'SurfaceNormals',AccessMode.ReadWrite)
    SurfaceType: str = proxy_property(str,'SurfaceType',AccessMode.ReadOnly)
    SurfTrimAssociativity: bool = proxy_property(Variant,'SurfTrimAssociativity',AccessMode.ReadWrite)
    UIsolineDensity: int = proxy_property(int,'UIsolineDensity',AccessMode.ReadWrite)
    VIsolineDensity: int = proxy_property(int,'VIsolineDensity',AccessMode.ReadWrite)
    WireframeType: AcWireframeType = proxy_property('AcWireframeType','WireframeType',AccessMode.ReadWrite)

    def Copy(self) -> AcadLoftedSurface:
        return AcadLoftedSurface(self._obj.Copy())


from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *

class AcadIDPair(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    IsCloned: bool = proxy_property(bool,'IsCloned',AccessMode.ReadOnly)
    IsOwnerXlated: bool = proxy_property(bool,'IsOwnerXlated',AccessMode.ReadOnly)
    IsPrimary: bool = proxy_property(bool,'IsPrimary',AccessMode.ReadOnly)
    Key: int = proxy_property(int,'Key',AccessMode.ReadOnly)


from __future__ import annotations
from .Base import *
from .Objects.AcadApplication import *
from .Proxy import *


class IAcadObject(AppObject):
    """The standard interface for a basic AutoCAD object."""
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    """Gets the Application object.

    object 
        Type: All objects 
        The object this property applies to. 
    Property Value
        Read-only: Yes 
        Type: Application 
        The Application object that is the owner of the object. 
    Remarks
        The Application object represents the application's frame controls and path settings, and provides the means to navigate down the object hierarchy. 
        Release Information
        Releases: AutoCAD 2000 through AutoCAD 2017 
        This property is no longer supported for use with the FileDependency and FileDependencies objects.
    """

    Document: AcadDocument = proxy_property('AcadDocument','Document',AccessMode.ReadOnly)
    """Gets the document (drawing) in which the object belongs.

    object 
        Type: All drawing objects, Block, Blocks, Dictionary, Dictionaries, DimStyle, DimStyles, Group, Groups, Layer, Layers, Layout, Layouts, Linetype, Linetypes, ModelSpace, PaperSpace, PlotConfiguration, PlotConfigurations, RegisteredApplication, RegisteredApplications, SectionManager, SectionSettings, SortentsTable, SubDMesh, TableStyle, TextStyle, TextStyles, UCS, UCSs, View, Views, Viewport, Viewports, XRecord 
        The objects this property applies to. 
    Property Value
        Read-only: Yes 
        Type: Document 
        The document (drawing) that contains the object. 
    Remarks
        No additional remarks. 
    """

    Handle: str = proxy_property(str,'Handle',AccessMode.ReadOnly)
    """Gets the handle of an object.

    object 
        Type: All drawing objects, AttributeReference, Block, Blocks, Dictionaries, Dictionary, DimStyle, DimStyles, Group, Groups, Layer, Layers, Layout, Layouts, Linetype, Linetypes, ModelSpace, PaperSpace, PlotConfiguration, PlotConfigurations, RegisteredApplication, RegisteredApplications, SectionManager, SectionSettings, SortentsTable, TableStyle, TextStyle, TextStyles, UCS, UCSs, View, Views, Viewport, Viewports, XRecord 
        The object this property applies to. 
    Property Value
        Read-only: Yes 
        Type: String 
        The handle of the entity. 
    Remarks
        An object ID and a unique handle are the two ways of referencing an object. A handle is persistent (stays the same) in a drawing for the lifetime of the object. 
        In general, use a handle unless you plan to work with certain ObjectARX functions that require an object ID. 
    """

    HasExtensionDictionary: bool = proxy_property(bool,'HasExtensionDictionary',AccessMode.ReadOnly)
    """Determines whether the object has an extension dictionary associated with it.

    object 
        Type: All drawing objects, AttributeReference, Block, Dictionary, DimStyle, Group, Layer, Layout, Linetype, Material, MLeaderStyle, PlotConfiguration, RegisteredApplication, SectionManager, SectionSettings, SortentsTable, TableStyle, TextStyle, UCS, View, Viewport, XRecord 
        The objects this property applies to. 
    Property Value
        Read-only: Yes 
        Type: Boolean 
        True: The object has an extension dictionary associated with it. 
        False: The object does not have an extension dictionary associated with it. 
    Remarks
        You can create an extension dictionary for an object, or query an existing extension dictionary by using the GetExtensionDictionary method. 
    """

    ObjectName: str = proxy_property(str,'ObjectName',AccessMode.ReadOnly)
    """Gets the AutoCAD class name of the object.

    object 
        Type: All drawing objects, AttributeReference, Block, Blocks, Dictionary, Dictionaries, Dimension, DimStyle, DimStyles, Group, Groups, Layer, Layers, Layout, Layouts, Linetype, Linetypes, Material, Materials, MLeaderStyle, ModelSpace, PaperSpace, PlotConfiguration, PlotConfigurations, RegisteredApplication, RegisteredApplications, SectionManager, SectionSettings, SortentsTable, SubDMeshEdge, SubDMeshFace, SubDMeshVertex, SubEntity, SubEntSolidEdge, SubEntSolidFace, SubEntSolidNode, SubEntSolidVertex, TableStyle, TextStyle, TextStyles, UCS, UCSs, View, Views, Viewport, Viewports, XRecord 
        The objects this property applies to. 
    Property Value
        Read-only: Yes 
        Type: String 
        The AutoCAD class name of an object. 
    Remarks
        No additional remarks. 
    """

    ObjectID: int = proxy_property(int,'ObjectID',AccessMode.ReadOnly)
    """Gets the object ID.

    object 
        Type: All drawing objects, AttributeReference, Block, Blocks, Dictionary, Dictionaries, Dimension, DimStyle, DimStyles, Group, Groups, Layer, Layers, Layout, Layouts, Linetype, Linetypes, Material, Materials, MLeaderStyle, ModelSpace, PaperSpace, PlotConfiguration, PlotConfigurations, RegisteredApplication, RegisteredApplications, SectionManager, SectionSettings, SortentsTable, TableStyle, TextStyle, TextStyles, UCS, UCSs, View, Views, Viewport, Viewports, XRecord 
        The objects this property applies to. 
    Property Value
        Read-only: Yes 
        Type: Long_PTR 
        The object ID of an object. 
    Remarks
        An object ID and a unique handle are both ways of referencing an object. 
        In general, use a handle unless you plan to work with certain ObjectARX functions that require an object ID. 
    """

    OwnerID: int = proxy_property(int,'OwnerID',AccessMode.ReadOnly)
    """Gets the object ID of the owner (parent) object.

    object 
        Type: All drawing objects, AttributeReference, Block, Blocks, Dictionary, Dictionaries, Dimension, DimStyle, DimStyles, Group, Groups, Layer, Layers, Layout, Layouts, Linetype, Linetypes, Material, Materials, MLeaderStyle, ModelSpace, PaperSpace, PlotConfiguration, PlotConfigurations, RegisteredApplication, RegisteredApplications, SectionManager, SectionSettings, SortentsTable, TableStyle, TextStyle, TextStyles, UCS, UCSs, View, Views, Viewport, Viewports, XRecord 
        The objects this property applies to. 
    Property Value
        Read-only: Yes 
        Type: Long_PTR 
        The object ID of an object's owner. 
    Remarks
        No additional remarks. 
    """

    def GetExtensionDictionary(self) -> AcadDictionary: 
        """Gets the extension dictionary associated with an object.

        object 
            Type: All drawing objects, AttributeReference, Block, Dictionary, Dimension, DimStyle, Group, Layer, Layout, Linetype, Material, MLeaderStyle, PlotConfiguration, RegisteredApplication, TableStyle, TextStyle, UCS, View, Viewport, XRecord
            The objects this method applies to. 
        Return Value (RetVal)
            Type: Dictionary 
            The extension dictionary for the object. 
        Remarks
            If an object does not have an extension dictionary, this method will create a new extension dictionary for that object and return it in the return value.
            You can query an object to see if it has an extension dictionary by using the HasExtensionDictionary property. 
        """
        return AcadDictionary(self._obj.GetExtensionDictionary())

    def GetXData(self, AppName: str = '') -> tuple:
        """Gets the extended data (XData) associated with an object.

        object 
            Type: All drawing objects, AttributeReference, Block, Dictionary, Dimension, DimStyle, Group, Layer, Layout, Linetype, Material, MLeaderStyle, PlotConfiguration, RegisteredApplication, TableStyle, TextStyle, UCS, View, Viewport, XRecord
            The objects this method applies to. 
        AppName 
            Access: Input-only 
            Type: String 
            A NULL string will return all the data attached to the object, regardless of the application that created it. Supplying an application name will return only the data that was created by the specified application.
        XDataType 
            Access: Output-only 
            Type: Variant (array of shorts) 
            An array of short integer values that represent the DXF group code values for each value in the extended data (XData). 
        XDataValue 
            Access: Output-only 
            Type: Variant (array of variants) 
            An array of values that make up the extended data (XData). 
        Return Value (RetVal)
            No return value. 
        Remarks
            Extended data is an example of instance-specific data created by applications written with ObjectARX or AutoLISP. This data can be added to any object. This data follows the object's definition data, and is maintained in the order that it was saved into the document. (AutoCAD maintains this information, but does not use it.)
        """
        XDataType, XDataValue = self._obj.GetXData(AppName)
        return XDataType, XDataValue

    # def SetXData(self, XDataType: Variant, XDataValue: Variant) -> None: 
    def SetXData(self, XDataType: vShortArray, XDataValue: vVariantArray) -> None: 
        """Sets the extended data (XData) associated with an object.

        object 
            Type: All drawing objects, AttributeReference, Block, Dictionary, Dimension, DimStyle, Group, Layer, Layout, Linetype, Material, MLeaderStyle, PlotConfiguration, RegisteredApplication, TableStyle, TextStyle, UCS, View, Viewport, XRecord 
            The object this method applies to. 
        XDataType 
            Access: Input-only 
            Type: Variant (array of short) 
            An array of short integer values that represent the DXF group code values for each value in the extended data (XData). 
        XDataValue 
            Access: Input-only 
            Type: Variant (array of variants) 
            An array of values that make up the extended data (XData). 
        Return Value (RetVal)
            No return value. 
        Remarks
            Extended data is an example of instance-specific data created by applications written with ObjectARX or AutoLISP. This data can be added to any entity. This data follows the entity's definition data and is maintained in the order in which it was saved into the document. (AutoCAD maintains this information but does not use it.)
        """
        self._obj.SetXData(XDataType(), XDataValue())

class AcadObject(IAcadObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    def Delete(self) -> None: 
        """Deletes a specified object or a set of saved layer settings.

        object 
            Type: All drawing objects, AttributeReference, Block, Dictionary, DimStyle, Group, Hyperlink, Layer, LayerStateManager, Layout, Linetype, Material, MLeaderStyle, PlotConfiguration, PopupMenuItem, RegisteredApplication, SelectionSet, TableStyle, TextStyle, Toolbar, ToolbarItem, UCS, View, Viewport, XRecord 
            The objects this method applies to. 
        Return Value (RetVal)
            No return value. 
        Remarks
            When you delete an object in a collection, all remaining items in the collection are reassigned a new index based on the current count. You should therefore avoid loops that delete an object while iterating through the collection. For example, the following VBA code will result in a runtime error:
            For i = 0 To ThisDrawing.Groups.Count - 1
                ThisDrawing.Groups.Item(i).Delete
            Next I
            Instead, use the following VBA code to delete all members in a collection: 
            For Each obj In ThisDrawing.Groups
                obj.Delete
            Next obj
            You can also use the following VBA code to delete a single member of a collection: 
            ThisDrawing.Groups.Item("group1").DeleteAn error will result if you attempt to delete a collection object. 
            ToolbarItem: You can only add or remove toolbar items when the toolbar is visible. 
            LayerStateManager: This object takes an argument, Name, which is a string representing the layer state to be deleted.
        """
        self._obj.Delete()

class IAcadObjectCollection(IAcadObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Count: int = proxy_property(int,'Count',AccessMode.ReadOnly)
    """Gets the number of items in the object.

    object 
        Type: All Collections, Block, Dictionary, Group, Materials, SectionManager, SelectionSet 
        The object this property applies to. 
    Property Value
        Read-only: Yes 
        Type: Integer 
        The number of items in the object. 
    Remarks
        No additional remarks. 
    Release Information
        Releases: AutoCAD 2000 through AutoCAD 2017 
        This property is no longer supported for use with the FileDependencies object.
    """

    def Item(self, Index: int | str):
        obj = self._obj.Item(Index)
        return AcadObject(obj)

    def __iter__(self):
        for item in self._obj:
            yield AcadObject(item)


from ..Base import *
from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadTextStyle(AcadObject):
    def __init__(self, obj) -> None: ...
    BigFontFile: str
    FontFile: str
    Height: float
    LastHeight: float
    Name: str
    ObliqueAngle: float
    TextGenerationFlag: AcTextGenerationFlag
    Width: float
    def GetFont(self) -> tuple[str, bool, bool, int, int]: ...
    def SetFont(self, Typeface: str, Bold: bool, Italic: bool, CharSet: int, PitchAndFamily: int) -> None: ...



from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadLoftedSurface(AcadEntity):
    def __init__(self, obj) -> None: ...
    Closed: bool
    EdgeExtensionDistances: Variant
    EndDraftAngle: float
    EndDraftMagnitude: int
    EndSmoothContinuity: int
    EndSmoothMagnitude: float
    MaintainAssociativity: int
    NumCrossSections: int
    NumGuidePaths: int
    Periodic: bool
    ShowAssociativity: bool
    StartDraftAngle: float
    StartDraftMagnitude: int
    StartSmoothContinuity: int
    StartSmoothMagnitude: float
    SurfaceNormals: int
    SurfaceType: str
    SurfTrimAssociativity: bool
    UIsolineDensity: int
    VIsolineDensity: int
    WireframeType: AcWireframeType
    def Copy(self) -> AcadLoftedSurface: ...
